#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <time.h>
#include "tests_body.h"
/**********************************************************************/
/************************ TEST SCENARIOS ******************************/

struct smb_test wrst;
const char* TLIST[16] = {"2.1.1", "2.1.2", "2.2.1", "2.2.2", "2.2.5", "2.2.6",
                        "2.2.7", "3.1.1", "3.1.2", "3.1.3", "3.1.4", "3.1.5",
                        "3.2.1", "3.2.2", "3.3.1", "3.3.2"};

void init_wr_test(int test_no, int variant)
{
  unsigned int vtag, i;

  /*default values for functional tests*/
  for(i=0; i<SMB_PORTS; ++i) {
    wrst.ptx[i].fsize    = 64;
    wrst.ptx[i].fsize_start= 64;
    wrst.ptx[i].fsize_step = 0;
    wrst.ptx[i].fsize_stop = 64;
    wrst.ptx[i].fsize_n  = 0;
    wrst.ptx[i].fload[0] = 10;
    wrst.ptx[i].fload_n  = 1;
    wrst.ptx[i].fnum     = T_2XX_FNUM;
  }
  wrst.test_type = TEST_FUNCTIONAL;

  /*loads and sizes for performance tests*/
  if (test_no >= TEST_3_1_1 && variant == VAR1) {
    wrst.ftime = 0;
    wrst.test_type = TEST_PERFORMANCE;
    for (i=0; i<SMB_PORTS; ++i) {
      /* size:   64 - 1522 with step of 1
       * loads:  10, 30, 50, 70, 80, 85, 90, 92, 95, 97, 99, 100
       * number: 10^9 */
      wrst.ptx[i].fsize      = 60;
      wrst.ptx[i].fsize_start= 60;
      wrst.ptx[i].fsize_step = 32; //1;
      wrst.ptx[i].fsize_stop = 128; //1522;
      wrst.ptx[i].fsize_n    = 0;
      wrst.ptx[i].fload[0]   = 10;
      wrst.ptx[i].fload[1]   = 30;
      wrst.ptx[i].fload[2]   = 50;
      wrst.ptx[i].fload[3]   = 70;
      wrst.ptx[i].fload[4]   = 80;
      wrst.ptx[i].fload[5]   = 85;
      wrst.ptx[i].fload[6]   = 90;
      wrst.ptx[i].fload[7]   = 92;
      wrst.ptx[i].fload[8]   = 95;
      wrst.ptx[i].fload[9]   = 97;
      wrst.ptx[i].fload[10]  = 99;
      wrst.ptx[i].fload[11]  = 100;
      wrst.ptx[i].fload_n    = 12;
      wrst.ptx[i].fnum       = 1000000; //1000000000;  //10^9
    }
  }
  if (test_no >= TEST_3_1_1 && variant == VAR2) {
    wrst.ftime = 20; //5*60;  //5 min
    wrst.test_type = TEST_PERFORMANCE;
    for (i=0; i<SMB_PORTS; ++i) {
      /* size:          64, 65, 700, 701, 1521, 1522
       * loads:         50, 80, 95, 100
       * time of burst: 5 min */
      wrst.ptx[i].fsize      = 0;
      wrst.ptx[i].fsize_start= 0;
      wrst.ptx[i].fsize_step = 0;
      wrst.ptx[i].fsize_stop = 0;
      wrst.ptx[i].fsize_tab[0] = 60;
      wrst.ptx[i].fsize_tab[1] = 61;
      wrst.ptx[i].fsize_tab[2] = 696;
      wrst.ptx[i].fsize_tab[3] = 697;
      wrst.ptx[i].fsize_tab[4] = 1513;
      wrst.ptx[i].fsize_tab[5] = 1514;
      wrst.ptx[i].fsize_n  = 6;
      wrst.ptx[i].fload[0] = 50; 
      wrst.ptx[i].fload[1] = 80; 
      wrst.ptx[i].fload[2] = 95; 
      wrst.ptx[i].fload[3] = 100; 
      wrst.ptx[i].fload_n  = 4; 
      wrst.ptx[i].fnum     = 0; 
    }
  }

  switch(test_no) {
    case TEST_2_1_1:
      /* 2.1.1  Unicast - one-to-one stream */
      /* TX:      p0->p2; p2 -> p0; 802.1d
       * RX_TRIG: p0; p2 -> smac 
       * RX:      p1; p3 */
      wrst.learning = 0;
      wrst.variants = 1;
      wrst.ptx[0].d_ports[0] = 2;
      wrst.ptx[2].d_ports[0] = 0;
      wrst.ptx[0].q_vid  = VID_NONE;
      wrst.ptx[2].q_vid  = VID_NONE;
      wrst.ptx[0].d_ports_n  = 1;
      wrst.ptx[1].d_ports_n  = 0;
      wrst.ptx[2].d_ports_n  = 1;
      wrst.ptx[3].d_ports_n  = 0;

      wrst.prx[0].s_port = 2;
      wrst.prx[2].s_port = 0;
      wrst.prx[1].s_port = RX_PALL;
      wrst.prx[3].s_port = RX_PALL;
      wrst.prx[0].trig2_len = 0;
      wrst.prx[1].trig2_len = 0;
      wrst.prx[2].trig2_len = 0;
      wrst.prx[3].trig2_len = 0;
      break;

    case TEST_2_1_2:
      /* 2.1.2 Unicast - many-to-many stream */
      /* TX:      p0->p1; p1->p0; p2->p3; p3->p2; 802.1d
       * RX_TRIG: p0; p1; p2; p3 */
      wrst.learning = 0;
      wrst.variants = 1;
      wrst.ptx[0].d_ports[0] = 1;
      wrst.ptx[0].d_ports[1] = 2;
      wrst.ptx[0].d_ports[2] = 3;

      wrst.ptx[1].d_ports[0] = 0;
      wrst.ptx[1].d_ports[1] = 2;
      wrst.ptx[1].d_ports[2] = 3;

      wrst.ptx[2].d_ports[0] = 0;
      wrst.ptx[2].d_ports[1] = 1;
      wrst.ptx[2].d_ports[2] = 3;

      wrst.ptx[3].d_ports[0] = 0;
      wrst.ptx[3].d_ports[1] = 1;
      wrst.ptx[3].d_ports[2] = 2;

      wrst.ptx[0].d_ports_n  = 3;
      wrst.ptx[1].d_ports_n  = 3;
      wrst.ptx[2].d_ports_n  = 3;
      wrst.ptx[3].d_ports_n  = 3;
      wrst.ptx[0].q_vid  = VID_NONE;
      wrst.ptx[1].q_vid  = VID_NONE;
      wrst.ptx[2].q_vid  = VID_NONE;
      wrst.ptx[3].q_vid  = VID_NONE;

      wrst.prx[0].s_port = RX_ALLUNI;
      wrst.prx[1].s_port = RX_ALLUNI;
      wrst.prx[2].s_port = RX_ALLUNI;
      wrst.prx[3].s_port = RX_ALLUNI;
      wrst.prx[0].trig2_len = 0;
      wrst.prx[1].trig2_len = 0;
      wrst.prx[2].trig2_len = 0;
      wrst.prx[3].trig2_len = 0;
      break;

    case TEST_2_2_1:
      /* 2.2.1 VLAN-limited forwarding */
      /* TX:      p0->p2; p2->p0; 802.1q
       * RX_TRIG: p0; p2 -> smac; vid; prio
       * RX:      p1; p3 */
      wrst.learning = 0;
      wrst.variants = 1;
      wrst.ptx[0].d_ports[0] = 2;
      wrst.ptx[2].d_ports[0] = 0;
      wrst.ptx[0].q_vid  = 1;
      wrst.ptx[0].q_pri  = 1;
      wrst.ptx[2].q_vid  = 1;
      wrst.ptx[2].q_pri  = 1;
      wrst.ptx[0].d_ports_n = 1;
      wrst.ptx[1].d_ports_n = 0;
      wrst.ptx[2].d_ports_n = 1;
      wrst.ptx[3].d_ports_n = 0;

      wrst.prx[0].s_port = 2;
      wrst.prx[2].s_port = 0;
      wrst.prx[1].s_port = RX_PALL;
      wrst.prx[3].s_port = RX_PALL;
      wrst.prx[0].trig2_len = 4;
      wrst.prx[2].trig2_len = 4;
      wrst.prx[1].trig2_len = 0;
      wrst.prx[3].trig2_len = 0;
      vtag = VLAN_TAG(1, 1);
      memcpy( wrst.prx[0].trig2_pat, &vtag, 4 );
      memcpy( wrst.prx[2].trig2_pat, &vtag, 4 );
      break;

    case TEST_2_2_2:
      /* 2.2.2 VLAN separation of traffic */
      /* TX:      p0; p2; 802.1q; broadcast
       * RX_TRIG: p0; p2 -> smac; vid; prio
       * RX:      p1; p3 */
      wrst.learning = 0;
      wrst.variants = 1;
      wrst.ptx[0].d_ports[0] = TX_PALL;
      wrst.ptx[2].d_ports[0] = TX_PALL;
      wrst.ptx[0].q_vid  = 1;
      wrst.ptx[0].q_pri  = 0;
      wrst.ptx[2].q_vid  = 1;
      wrst.ptx[2].q_pri  = 0;
      wrst.ptx[0].d_ports_n = 1;
      wrst.ptx[1].d_ports_n = 0;
      wrst.ptx[2].d_ports_n = 1;
      wrst.ptx[3].d_ports_n = 0;

      wrst.prx[0].s_port = 2;
      wrst.prx[2].s_port = 0;
      wrst.prx[1].s_port = RX_PALL;
      wrst.prx[3].s_port = RX_PALL;
      wrst.prx[0].trig2_len = 4;
      wrst.prx[2].trig2_len = 4;
      wrst.prx[1].trig2_len = 0;
      wrst.prx[3].trig2_len = 0;
      vtag = VLAN_TAG(1, 0);
      memcpy( wrst.prx[0].trig2_pat, &vtag, 4 );
      memcpy( wrst.prx[2].trig2_pat, &vtag, 4 );
      break;

    case TEST_2_2_5:
      /* 2.2.5 Tagging */
      /* TX:      p0->p2; p2->p0; 802.1d
       * RX_TRIG: p0; p2 -> smac; vid; prio
       * RX:      p1; p3 */
      wrst.learning = 0;
      wrst.variants = 1;
      wrst.ptx[0].d_ports[0] = 2;
      wrst.ptx[2].d_ports[0] = 0;
      wrst.ptx[0].q_vid  = VID_NONE;
      wrst.ptx[2].q_vid  = VID_NONE;
      wrst.ptx[0].d_ports_n = 1;
      wrst.ptx[1].d_ports_n = 0;
      wrst.ptx[2].d_ports_n = 1;
      wrst.ptx[3].d_ports_n = 0;

      wrst.prx[0].s_port = 2;
      wrst.prx[2].s_port = 0;
      wrst.prx[1].s_port = RX_PALL;
      wrst.prx[3].s_port = RX_PALL;
      wrst.prx[0].trig2_len = 4;
      wrst.prx[2].trig2_len = 4;
      wrst.prx[1].trig2_len = 0;
      wrst.prx[3].trig2_len = 0;
      vtag = VLAN_TAG(1, 3);
      memcpy( wrst.prx[0].trig2_pat, &vtag, 4 );
      memcpy( wrst.prx[2].trig2_pat, &vtag, 4 );
      break;

    case TEST_2_2_6:
      /* 2.2.6 Untagging */
      /* TX:      p0->p2; p2->p0; 802.1q
       * RX_TRIG: p0; p2 -> smac; ethtype
       * RX:      p1; p3 */
      wrst.learning = 0;
      wrst.variants = 1;
      wrst.ptx[0].d_ports[0] = 2;
      wrst.ptx[2].d_ports[0] = 0;
      wrst.ptx[0].q_vid  = 1;
      wrst.ptx[0].q_pri  = 0;
      wrst.ptx[2].q_vid  = 1;
      wrst.ptx[2].q_pri  = 0;
      wrst.ptx[0].d_ports_n = 1;
      wrst.ptx[1].d_ports_n = 0;
      wrst.ptx[2].d_ports_n = 1;
      wrst.ptx[3].d_ports_n = 0;

      wrst.prx[0].s_port = 2;
      wrst.prx[2].s_port = 0;
      wrst.prx[1].s_port = RX_PALL;
      wrst.prx[3].s_port = RX_PALL;
      wrst.prx[0].trig2_len = 2;
      wrst.prx[2].trig2_len = 2;
      wrst.prx[1].trig2_len = 0;
      wrst.prx[3].trig2_len = 0;
      vtag = NOVLAN_TAG;
      memcpy( wrst.prx[0].trig2_pat, &vtag, 2 );
      memcpy( wrst.prx[2].trig2_pat, &vtag, 2 );
      break;

    case TEST_2_2_7:
      /* 2.2.7 Snake test */
      /* TX:  p0->p1; p1->p0; p2->p3; p3->p2
       * RX_TRIG: p0; p1; p2; p3 -> smac; ethtype */
      wrst.learning = 0;
      wrst.variants = 1;
      wrst.ptx[0].d_ports[0] = 1;
      wrst.ptx[1].d_ports[0] = 0;
      wrst.ptx[2].d_ports[0] = 3;
      wrst.ptx[3].d_ports[0] = 2;
      wrst.ptx[0].q_vid  = VID_NONE;
      wrst.ptx[1].q_vid  = VID_NONE;
      wrst.ptx[2].q_vid  = VID_NONE;
      wrst.ptx[3].q_vid  = VID_NONE;
      wrst.ptx[0].d_ports_n = 1;
      wrst.ptx[1].d_ports_n = 1;
      wrst.ptx[2].d_ports_n = 1;
      wrst.ptx[3].d_ports_n = 1;

      wrst.prx[0].s_port = 1;
      wrst.prx[1].s_port = 0;
      wrst.prx[2].s_port = 3;
      wrst.prx[3].s_port = 2;
      wrst.prx[0].trig2_len = 2;
      wrst.prx[1].trig2_len = 2;
      wrst.prx[2].trig2_len = 2;
      wrst.prx[3].trig2_len = 2;
      vtag = NOVLAN_TAG;
      memcpy( wrst.prx[0].trig2_pat, &vtag, 2 );
      memcpy( wrst.prx[1].trig2_pat, &vtag, 2 );
      memcpy( wrst.prx[2].trig2_pat, &vtag, 2 );
      memcpy( wrst.prx[3].trig2_pat, &vtag, 2 );
      break;

    /***********************************************/ 
    /*             PERFORMANCE TESTS               */
    /***********************************************/ 
    case TEST_3_1_1:
      wrst.learning = 1;
      wrst.variants = 2;
      wrst.ptx[0].d_ports[0] = 2;
      wrst.ptx[2].d_ports[0] = 0;
      wrst.ptx[1].d_ports[0] = 3;
      wrst.ptx[3].d_ports[0] = 1;
      wrst.ptx[0].q_vid  = VID_NONE;
      wrst.ptx[1].q_vid  = VID_NONE;
      wrst.ptx[2].q_vid  = VID_NONE;
      wrst.ptx[3].q_vid  = VID_NONE;
      wrst.ptx[0].d_ports_n = 1;
      wrst.ptx[1].d_ports_n = 1;
      wrst.ptx[2].d_ports_n = 1;
      wrst.ptx[3].d_ports_n = 1;

      wrst.prx[0].s_port = 2;
      wrst.prx[2].s_port = 0;
      wrst.prx[1].s_port = 3;
      wrst.prx[3].s_port = 1;
      wrst.prx[0].trig2_len = 0;
      wrst.prx[1].trig2_len = 0;
      wrst.prx[2].trig2_len = 0;
      wrst.prx[3].trig2_len = 0;
      break;

    case TEST_3_1_2:
      /* 3.1.2 Unicast one-to-one streams */
      /* TX: p0->p1; p1->p0; p2->p3; p3->p2
       * RX_TRIG: p0; p1; p2; p3 -> smac */
      wrst.learning = 1;
      wrst.variants = 2;
      wrst.ptx[0].d_ports[0] = 1;
      wrst.ptx[1].d_ports[0] = 0;
      wrst.ptx[2].d_ports[0] = 3;
      wrst.ptx[3].d_ports[0] = 2;
      wrst.ptx[0].q_vid = VID_NONE;
      wrst.ptx[1].q_vid = VID_NONE;
      wrst.ptx[2].q_vid = VID_NONE;
      wrst.ptx[3].q_vid = VID_NONE;
      wrst.ptx[0].d_ports_n = 1;
      wrst.ptx[1].d_ports_n = 1;
      wrst.ptx[2].d_ports_n = 1;
      wrst.ptx[3].d_ports_n = 1;

      wrst.prx[0].s_port = 1;
      wrst.prx[1].s_port = 0;
      wrst.prx[2].s_port = 3;
      wrst.prx[3].s_port = 2;
      wrst.prx[0].trig2_len = 0;
      wrst.prx[1].trig2_len = 0;
      wrst.prx[2].trig2_len = 0;
      wrst.prx[3].trig2_len = 0;
      break;

    case TEST_3_1_3:
      /* 3.1.3 Unicast many-to-many streams */
      /* TX: p0->p1,p2,p3; p1->p0,p2,p3; p2->p0,p1,p3; p3->p0,p1,p2
       * RX_TRIG: p0; p1; p2; p3 */
      wrst.learning = 1;
      wrst.variants = 2;
      wrst.ptx[0].d_ports[0] = 1;
      wrst.ptx[0].d_ports[1] = 2;
      wrst.ptx[0].d_ports[2] = 3;

      wrst.ptx[1].d_ports[0] = 0;
      wrst.ptx[1].d_ports[1] = 2;
      wrst.ptx[1].d_ports[2] = 3;

      wrst.ptx[2].d_ports[0] = 0;
      wrst.ptx[2].d_ports[1] = 1;
      wrst.ptx[2].d_ports[2] = 3;

      wrst.ptx[3].d_ports[0] = 0;
      wrst.ptx[3].d_ports[1] = 1;
      wrst.ptx[3].d_ports[2] = 2;

      wrst.ptx[0].d_ports_n  = 3;
      wrst.ptx[1].d_ports_n  = 3;
      wrst.ptx[2].d_ports_n  = 3;
      wrst.ptx[3].d_ports_n  = 3;
      wrst.ptx[0].q_vid  = VID_NONE;
      wrst.ptx[1].q_vid  = VID_NONE;
      wrst.ptx[2].q_vid  = VID_NONE;
      wrst.ptx[3].q_vid  = VID_NONE;

      wrst.prx[0].s_port = RX_ALLUNI;
      wrst.prx[1].s_port = RX_ALLUNI;
      wrst.prx[2].s_port = RX_ALLUNI;
      wrst.prx[3].s_port = RX_ALLUNI;
      wrst.prx[0].trig2_len = 0;
      wrst.prx[1].trig2_len = 0;
      wrst.prx[2].trig2_len = 0;
      wrst.prx[3].trig2_len = 0;
      break;

    case TEST_3_1_4:
      /* 3.1.4 VLAN, broadcast */
      /* TX: p0; p1; p2; p3 -> broadcast, 802.1q
       * RX_TRIG: p0; p1; p2; p3 -> smac, vid, prio */
      wrst.learning = 1;
      wrst.variants = 2;
      wrst.ptx[0].d_ports[0] = TX_PALL;
      wrst.ptx[1].d_ports[0] = TX_PALL;
      wrst.ptx[2].d_ports[0] = TX_PALL;
      wrst.ptx[3].d_ports[0] = TX_PALL;
      wrst.ptx[0].q_vid = 1;
      wrst.ptx[1].q_vid = 2;
      wrst.ptx[2].q_vid = 1;
      wrst.ptx[3].q_vid = 2;
      wrst.ptx[0].q_pri = 0;
      wrst.ptx[1].q_pri = 0;
      wrst.ptx[2].q_pri = 0;
      wrst.ptx[3].q_pri = 0;
      wrst.ptx[0].d_ports_n = 1;
      wrst.ptx[1].d_ports_n = 1;
      wrst.ptx[2].d_ports_n = 1;
      wrst.ptx[3].d_ports_n = 1;

      wrst.prx[0].s_port = 2;
      wrst.prx[2].s_port = 0;
      wrst.prx[1].s_port = 3;
      wrst.prx[3].s_port = 1;
      wrst.prx[0].trig2_len = 4;
      wrst.prx[1].trig2_len = 4;
      wrst.prx[2].trig2_len = 4;
      wrst.prx[3].trig2_len = 4;
      vtag = VLAN_TAG(1, 0);
      memcpy( wrst.prx[0].trig2_pat, &vtag, 4 );
      memcpy( wrst.prx[2].trig2_pat, &vtag, 4 );
      vtag = VLAN_TAG(2, 0);
      memcpy( wrst.prx[1].trig2_pat, &vtag, 4 );
      memcpy( wrst.prx[3].trig2_pat, &vtag, 4 );
      break;

    case TEST_3_1_5:
      /* 3.1.5 VLAG, broadcast, tagging/untagging */
      /* TX: p0->p2; p2->p0; p1->p3; p3->p1
       * RX_TRIG: p0; p1; p2; p3 -> smac */
      wrst.learning = 1;
      wrst.variants = 2;
      wrst.ptx[0].d_ports[0] = TX_PALL;
      wrst.ptx[1].d_ports[0] = TX_PALL;
      wrst.ptx[2].d_ports[0] = TX_PALL;
      wrst.ptx[3].d_ports[0] = TX_PALL;
      wrst.ptx[0].q_vid = VID_NONE;
      wrst.ptx[1].q_vid = VID_NONE;
      wrst.ptx[2].q_vid = VID_NONE;
      wrst.ptx[3].q_vid = VID_NONE;
      wrst.ptx[0].d_ports_n = 1;
      wrst.ptx[1].d_ports_n = 1;
      wrst.ptx[2].d_ports_n = 1;
      wrst.ptx[3].d_ports_n = 1;

      wrst.prx[0].s_port = 2;
      wrst.prx[2].s_port = 0;
      wrst.prx[1].s_port = 3;
      wrst.prx[3].s_port = 1;
      wrst.prx[0].trig2_len = 0;
      wrst.prx[1].trig2_len = 0;
      wrst.prx[2].trig2_len = 0;
      wrst.prx[3].trig2_len = 0;
      break;

    case TEST_3_2_1:
      /* 3.2.1 Snake - Uniformly distributed load on all ports */
      /* TX: p0->p1; p1->p0; p2->p3; p3->p2
       * RX_TRIG: p0; p1; p2; p3 -> smac, ethtype */
      wrst.learning = 1;
      wrst.variants = 2;
      wrst.ptx[0].d_ports[0] = 1;
      wrst.ptx[1].d_ports[0] = 0;
      wrst.ptx[2].d_ports[0] = 3;
      wrst.ptx[3].d_ports[0] = 2;
      wrst.ptx[0].q_vid = VID_NONE;
      wrst.ptx[1].q_vid = VID_NONE;
      wrst.ptx[2].q_vid = VID_NONE;
      wrst.ptx[3].q_vid = VID_NONE;
      wrst.ptx[0].d_ports_n = 1;
      wrst.ptx[1].d_ports_n = 1;
      wrst.ptx[2].d_ports_n = 1;
      wrst.ptx[3].d_ports_n = 1;

      wrst.prx[0].s_port = 1;
      wrst.prx[1].s_port = 0;
      wrst.prx[2].s_port = 3;
      wrst.prx[3].s_port = 2;
      wrst.prx[0].trig2_len = 2;
      wrst.prx[1].trig2_len = 2;
      wrst.prx[2].trig2_len = 2;
      wrst.prx[3].trig2_len = 2;
      vtag = NOVLAN_TAG;
      memcpy( wrst.prx[0].trig2_pat, &vtag, 2 );
      memcpy( wrst.prx[1].trig2_pat, &vtag, 2 );
      memcpy( wrst.prx[2].trig2_pat, &vtag, 2 );
      memcpy( wrst.prx[3].trig2_pat, &vtag, 2 );
      break;

    case TEST_3_2_2:
      break;

    case TEST_3_3_1:
      break;

    case TEST_3_3_2:
      break;
  }
}

int run_test( HTCountStructure *cntrs, int fload_idx, char mac[][6] )
{
  int i, j, rx_pmask = 0, tx_pmask = 0;
  struct smb_port_tx *p_tx = NULL;
  struct smb_port_rx *p_rx = NULL;
  
  //reset_ports(SMB_P0 | SMB_P1 | SMB_P2 | SMB_P3);

  ///* optional learning */
  //if (wrst.learning)
  //  smb_learning(SMB_PALL, mac, 3);

  /* setup required streams */
  for (i=0; i<SMB_PORTS; ++i) {
    p_tx = &(wrst.ptx[i]);

    //if (p_tx->d_ports_n > 0) {
    //  tx_pmask |= PORT_TO_PMASK(i);
    //  setup_stream(i, mac[p_tx->d_ports[0]], mac[i], p_tx->fsize, p_tx->fnum,
    //      p_tx->fload[0]/100.0, p_tx->q_pri, p_tx->q_vid);
    //}
    for (j=0; j< p_tx->d_ports_n; ++j) {
      setup_stream(i, mac[p_tx->d_ports[j]], mac[i], p_tx->fsize, p_tx->fnum,
          p_tx->fload[fload_idx]/100.0/p_tx->d_ports_n, p_tx->q_pri, p_tx->q_vid, j);
    }
    if(p_tx->d_ports_n>0) {
      tx_pmask |= PORT_TO_PMASK(i);
      setup_stream(i, mac[p_tx->d_ports[j]], mac[i], p_tx->fsize, p_tx->fnum,
          p_tx->fload[fload_idx]/100.0/p_tx->d_ports_n, p_tx->q_pri, p_tx->q_vid, -1);
    }
  }

  /* setup required triggers */
  for (i=0; i<SMB_PORTS; ++i) {
    p_rx = &(wrst.prx[i]);

    /* TRIG1 - SMAC or DMAC */
    if (p_rx->s_port == RX_ALLUNI)
      smb_set_trigger( mac[i], i, 0, 6, HTTRIGGER_1 );
    else if (p_rx->s_port != RX_PALL)
      smb_set_trigger( mac[p_rx->s_port], i, 6, 6, HTTRIGGER_1 );
    /* TRIG2 - Ethtype */
    if (p_rx->trig2_len > 0)
      smb_set_trigger( p_rx->trig2_pat, i, 12, p_rx->trig2_len, HTTRIGGER_2);
  }

  clear_allcntrs(SMB_PALL);

  ////wait for link up
  rx_pmask = 0;
  for (i=0; i<SMB_PORTS; ++i) {
    p_rx = &(wrst.prx[i]);
    if ((p_rx->s_port >= 0 && p_rx->s_port <=3) || p_rx->s_port == RX_PALL)
      rx_pmask |= PORT_TO_PMASK(i);
  }
  wait_linkup(rx_pmask);

  //generate traffic
  run_traffic(tx_pmask, TRAFFIC_START);
  //NSDelay(1);

  wait_test_done(tx_pmask, wrst.ftime);

  if(wrst.ftime > 0) {
    run_traffic(tx_pmask, TRAFFIC_STOP);
    wait_test_done(tx_pmask, 0);
  }

  get_allcntrs(SMB_PALL, cntrs);

  return 0;
}

int anl_result( HTCountStructure *cntrs, int testno)
{
  int req_num;

  switch (testno) {
    case TEST_2_1_1:
      /* All frames sent from p0 received on p2
       * All frames sent from p2 received on p0
       * Limited number of frames may be received on p1; p3 due to learning */
      if ( (cntrs[0].TmtPkt == T_2XX_FNUM && cntrs[2].RcvPkt == T_2XX_FNUM && cntrs[2].RcvTrig == T_2XX_FNUM) &&
           (cntrs[2].TmtPkt == T_2XX_FNUM && cntrs[0].RcvPkt == T_2XX_FNUM && cntrs[0].RcvTrig == T_2XX_FNUM) &&
           (cntrs[1].RcvPkt < T_2XX_UNRECOG_THR) && (cntrs[3].RcvPkt < T_2XX_UNRECOG_THR))
        return TEST_PASSED;

      if ( (cntrs[0].TmtPkt == T_2XX_FNUM && cntrs[2].RcvPkt == T_2XX_FNUM && cntrs[2].RcvTrig == T_2XX_FNUM) &&
           (cntrs[2].TmtPkt == T_2XX_FNUM && cntrs[0].RcvPkt == T_2XX_FNUM && cntrs[0].RcvTrig == T_2XX_FNUM) )
        return TEST_WARNING;

      break;
      
    case TEST_2_1_2:
      req_num = (SMB_PORTS-1)*T_2XX_FNUM;
      /* Each of the ports got frames addressed to it from all other ports
       * Limited number of frames addressed to other ports may be received due to learning */
      if ( (cntrs[0].TmtPkt == req_num && cntrs[0].RcvTrig == req_num && cntrs[0].RcvPkt <= req_num + T_2XX_UNRECOG_THR) &&
           (cntrs[1].TmtPkt == req_num && cntrs[1].RcvTrig == req_num && cntrs[1].RcvPkt <= req_num + T_2XX_UNRECOG_THR) &&
           (cntrs[2].TmtPkt == req_num && cntrs[2].RcvTrig == req_num && cntrs[2].RcvPkt <= req_num + T_2XX_UNRECOG_THR) &&
           (cntrs[3].TmtPkt == req_num && cntrs[3].RcvTrig == req_num && cntrs[3].RcvPkt <= req_num + T_2XX_UNRECOG_THR) )
        return TEST_PASSED;

      if ( (cntrs[0].TmtPkt == req_num && cntrs[0].RcvTrig == req_num) &&
           (cntrs[1].TmtPkt == req_num && cntrs[1].RcvTrig == req_num) &&
           (cntrs[2].TmtPkt == req_num && cntrs[2].RcvTrig == req_num) &&
           (cntrs[3].TmtPkt == req_num && cntrs[3].RcvTrig == req_num) )
        return TEST_WARNING;
      break;

    case TEST_2_2_7:
      /* All frames sent from p0 received on p1
       * All frames sent from p1 received on p0
       * All frames sent from p2 received on p3
       * All frames sent from p3 received on p2 */
      if ( (cntrs[0].TmtPkt == T_2XX_FNUM && cntrs[1].RcvPkt == T_2XX_FNUM && cntrs[1].RcvTrig == T_2XX_FNUM) &&
           (cntrs[1].TmtPkt == T_2XX_FNUM && cntrs[0].RcvPkt == T_2XX_FNUM && cntrs[0].RcvTrig == T_2XX_FNUM) &&
           (cntrs[2].TmtPkt == T_2XX_FNUM && cntrs[3].RcvPkt == T_2XX_FNUM && cntrs[3].RcvTrig == T_2XX_FNUM) &&
           (cntrs[3].TmtPkt == T_2XX_FNUM && cntrs[2].RcvPkt == T_2XX_FNUM && cntrs[2].RcvTrig == T_2XX_FNUM) )
        return TEST_PASSED;

      if ( (cntrs[0].TmtPkt == T_2XX_FNUM && cntrs[1].RcvPkt >= T_2XX_FNUM && cntrs[1].RcvTrig == T_2XX_FNUM) &&
           (cntrs[1].TmtPkt == T_2XX_FNUM && cntrs[0].RcvPkt >= T_2XX_FNUM && cntrs[0].RcvTrig == T_2XX_FNUM) &&
           (cntrs[2].TmtPkt == T_2XX_FNUM && cntrs[3].RcvPkt >= T_2XX_FNUM && cntrs[3].RcvTrig == T_2XX_FNUM) &&
           (cntrs[3].TmtPkt == T_2XX_FNUM && cntrs[2].RcvPkt >= T_2XX_FNUM && cntrs[2].RcvTrig == T_2XX_FNUM) )
        return TEST_WARNING;

      break;

    case TEST_2_2_1:
    case TEST_2_2_5:
    case TEST_2_2_6:
      /* All frames sent from p0 received on p2
       * All frames sent from p2 received on p0
       * No frames received on ports other than p0; p2 */
      if ( (cntrs[0].TmtPkt == T_2XX_FNUM && cntrs[2].RcvPkt == T_2XX_FNUM && cntrs[2].RcvTrig == T_2XX_FNUM) &&
           (cntrs[2].TmtPkt == T_2XX_FNUM && cntrs[0].RcvPkt == T_2XX_FNUM && cntrs[0].RcvTrig == T_2XX_FNUM) &&
           (cntrs[1].RcvPkt == 0 && cntrs[3].RcvPkt == 0) )
        return TEST_PASSED;
      break;

    case TEST_2_2_2:
      /* All frames sent from p0 dropped
       * All frames sent from p2 received only on p0
       * No frames received on ports other than p0 */
      if ( (cntrs[0].TmtPkt == T_2XX_FNUM && cntrs[2].RcvPkt == 0 && cntrs[2].RcvTrig == 0) &&
           (cntrs[2].TmtPkt == T_2XX_FNUM && cntrs[0].RcvPkt == T_2XX_FNUM && cntrs[0].RcvTrig == T_2XX_FNUM) &&
           (cntrs[1].RcvPkt == 0 && cntrs[2].RcvPkt == 0 && cntrs[3].RcvPkt == 0) )
        return TEST_PASSED;
      break;

    case TEST_3_1_1:
      /* All frames sent from p0 received on p2
       * All frames sent from p2 received on p0
       * All frames sent from p1 received on p3
       * All frames sent from p3 received on p1 */
      if ( (cntrs[0].TmtPkt == cntrs[2].RcvPkt && cntrs[0].TmtPkt == cntrs[2].RcvTrig) &&
           (cntrs[2].TmtPkt == cntrs[0].RcvPkt && cntrs[2].TmtPkt == cntrs[0].RcvTrig) &&
           (cntrs[1].TmtPkt == cntrs[3].RcvPkt && cntrs[1].TmtPkt == cntrs[3].RcvTrig) &&
           (cntrs[3].TmtPkt == cntrs[1].RcvPkt && cntrs[3].TmtPkt == cntrs[1].RcvTrig) )
        return TEST_PASSED;
      break;
  }

  return TEST_FAILED;
}

int run_and_analyze(int test_no, FILE *logfile)
{
  char mac[SMB_PORTS+1][6] = {{0x3c, 0x47, 0x0e, 0x04, 0x05, 0x06},
                              {0x3c, 0x47, 0x0e, 0x14, 0x15, 0x16},
                              {0x3c, 0x47, 0x0e, 0x24, 0x25, 0x26},
                              {0x3c, 0x47, 0x0e, 0x34, 0x35, 0x36},
                              {0xff, 0xff, 0xff, 0xff, 0xff, 0xff}};
  HTCountStructure cntrs[SMB_PORTS];
  unsigned long exp_rxf[SMB_PORTS];
  int result, i, j, fload_n, i_size, size_done, i_var;
  int force_break = 0;
  time_t t_start, t_stop;

  printf("Configuring WR Switch...\n");
  wrs_config("127.0.0.1", test_no);
  reset_ports(SMB_P0 | SMB_P1 | SMB_P2 | SMB_P3);

  init_wr_test(test_no, 0);
  for(i_var=0; i_var<wrst.variants; ++i_var) {

    if(i_var == 0) {
      printf("\n\n------ TEST %s ------\n", TLIST[test_no]);
      fprintf(logfile, "\n\n------ TEST %s ------\n", TLIST[test_no]);
    } else {
      printf("\n\n------ TEST %s.%d ------\n", TLIST[test_no], i_var);
      fprintf(logfile, "\n\n------ TEST %s.%d ------\n", TLIST[test_no], i_var);
    }
    init_wr_test(test_no, i_var);
    printf("Running test...");
    fflush(stdout);

    /*searching for a non-zero load on ports*/
    for(i=0; i<SMB_PORTS; ++i) {
      if(wrst.ptx[i].fload_n > 0)
        fload_n = wrst.ptx[i].fload_n;
      else if(fload_n > 0 && wrst.ptx[i].fload_n != fload_n) {
        printf("Configuration error, all ports should have the same number of fload steps\n");
        return -1;
      }
    }

    /* optional learning */
    if (wrst.learning)
      smb_learning(SMB_PALL, mac, 3);

    for(i=0; i<fload_n; ++i) {
      size_done = 0;
      i_size = 0;
      for(j=0; j<SMB_PORTS; ++j) {
        if(wrst.ptx[j].fsize_n > 0)
          wrst.ptx[j].fsize = wrst.ptx[j].fsize_tab[0];
        else
          wrst.ptx[j].fsize = wrst.ptx[j].fsize_start;
      }
      while(!size_done) {
        t_start = time(NULL);
        printf("\nload %d frame size %d B...", wrst.ptx[0].fload[i], wrst.ptx[0].fsize);
        fflush(stdout);
        fprintf(logfile, "\nload %d frame size %d B...", wrst.ptx[0].fload[i], wrst.ptx[0].fsize);
        run_test(cntrs, i, mac);
        exp_rx_frames(&wrst, cntrs, exp_rxf);
        printf("expected RX frames: p0 = %lu, p1 = %lu, p2 = %lu, p3 = %lu\n",
            exp_rxf[0], exp_rxf[1], exp_rxf[2], exp_rxf[3]);
        result = anl_result(cntrs, test_no);
        if(wrst.test_type == TEST_PERFORMANCE)
          show_floss(stdout, cntrs, exp_rxf);

        t_stop = time(NULL);
        switch (result) {
          case TEST_PASSED:
            printf("Passed (time %u)\n", (unsigned)(t_stop - t_start));
            fprintf(logfile, "Passed (time %u)\n", (unsigned)(t_stop - t_start));
            break;
          case TEST_WARNING:
            printf("Passed but with WARNING (time %u)\n", (unsigned)(t_stop - t_start));
            fprintf(logfile, "Passed but with WARNING (time %u)\n", (unsigned)(t_stop - t_start));
            break;
          case TEST_FAILED:
            printf("Failed (time %u)\n", (unsigned)(t_stop - t_start));
            fprintf(logfile, "Failed (time %u)\n", (unsigned)(t_stop - t_start));
            break;
        }

        show_allcntrs(logfile, SMB_PALL, cntrs);
        fflush(logfile);
        //show_allcntrs(stdout, SMB_PALL, cntrs);
       

        ++i_size;
        for(j=0; j<SMB_PORTS; ++j) {
          /* stop condition */
          if ( (wrst.ptx[j].fsize_n > 0 && i_size >= wrst.ptx[j].fsize_n) ||
               (wrst.ptx[j].fsize_n == 0 && wrst.ptx[j].fsize >= wrst.ptx[j].fsize_stop) )
            size_done = 1;
          /* next test */
          if (wrst.ptx[j].fsize_n > 0 && i_size < wrst.ptx[j].fsize_n)
            wrst.ptx[j].fsize = wrst.ptx[j].fsize_tab[i_size];
          else if (wrst.ptx[j].fsize_n == 0)
            wrst.ptx[j].fsize += wrst.ptx[j].fsize_step;
        }

        force_break = detect_stop();
        if(force_break > 0) break;
      }

      if(force_break > 0) break;
    }
  }

  return result;
}
